=== modified file 'DBUS-API'
--- DBUS-API	2011-11-26 23:08:17 +0000
+++ DBUS-API	2011-12-31 23:05:34 +0000
@@ -157,8 +157,8 @@
 
 * Copyright
 
-    Copyright © 2010-2011 Teddy Hogeborn
-    Copyright © 2010-2011 Björn Påhlsson
+    Copyright © 2010-2012 Teddy Hogeborn
+    Copyright © 2010-2012 Björn Påhlsson
   
 ** License:
    

=== modified file 'TODO'
--- TODO	2011-12-25 00:43:51 +0000
+++ TODO	2011-12-27 14:15:40 +0000
@@ -69,6 +69,13 @@
 ** TODO Allow %%(checker)s as a runtime expansion
 ** TODO Use python-tlslite?
 ** TODO D-Bus AddClient() method on server object
+** TODO Use org.freedesktop.DBus.Method.NoReply annotation on async methods.
+** TODO Emit [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-properties][org.freedesktop.DBus.Properties.PropertiesChanged]] signal
+   TODO Deprecate se.recompile.Mandos.Client.PropertyChanged - annotate!
+   TODO Can use "invalidates" annotation to also emit on changed secret.
+** TODO Support [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager][org.freedesktop.DBus.ObjectManager]] interface on server object
+   Deprecate methods GetAllClients(), GetAllClientsWithProperties()
+   and signals ClientAdded and ClientRemoved.
 
 * mandos.xml
 ** Add mandos contact info in manual pages
@@ -87,8 +94,7 @@
 ** Urwid client data displayer
    Better view of client data in the listing
 *** Properties popup
-** Nicer crashes. Stack traces Messes up shell.
-*** Print a nice "We are sorry" message, save stack trace to log.
+** Print a nice "We are sorry" message, save stack trace to log.
 ** Show timeout countdown for approval
 
 * mandos-keygen

=== modified file 'debian/copyright'
--- debian/copyright	2011-10-10 20:29:58 +0000
+++ debian/copyright	2011-12-31 23:05:34 +0000
@@ -4,8 +4,8 @@
 Source: <http://www.recompile.se/mandos>
 
 Files: *
-Copyright: Copyright © 2008-2011 Teddy Hogeborn
-Copyright: Copyright © 2008-2011 Björn Påhlsson
+Copyright: Copyright © 2008-2012 Teddy Hogeborn
+Copyright: Copyright © 2008-2012 Björn Påhlsson
 License: GPL-3+
     This program is free software: you can redistribute it and/or
     modify it under the terms of the GNU General Public License as

=== modified file 'intro.xml'
--- intro.xml	2011-10-05 16:00:56 +0000
+++ intro.xml	2011-12-31 23:05:34 +0000
@@ -1,7 +1,7 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -31,6 +31,7 @@
     </authorgroup>
     <copyright>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos'
--- mandos	2011-12-23 20:00:52 +0000
+++ mandos	2011-12-31 23:05:34 +0000
@@ -11,8 +11,8 @@
 # "AvahiService" class, and some lines in "main".
 # 
 # Everything else is
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -142,7 +142,8 @@
         self.gnupg.options.meta_interactive = False
         self.gnupg.options.homedir = self.tempdir
         self.gnupg.options.extra_args.extend(['--force-mdc',
-                                              '--quiet'])
+                                              '--quiet',
+                                              '--no-use-agent'])
     
     def __enter__(self):
         return self
@@ -459,9 +460,12 @@
 
     @staticmethod
     def config_parser(config):
-        """ Construct a new dict of client settings of this form:
+        """Construct a new dict of client settings of this form:
         { client_name: {setting_name: value, ...}, ...}
-        with exceptions for any special settings as defined above"""
+        with exceptions for any special settings as defined above.
+        NOTE: Must be a pure function. Must return the same result
+        value given the same arguments.
+        """
         settings = {}
         for client_name in config.sections():
             section = dict(config.items(client_name))
@@ -471,7 +475,8 @@
             # Reformat values from string types to Python types
             client["approved_by_default"] = config.getboolean(
                 client_name, "approved_by_default")
-            client["enabled"] = config.getboolean(client_name, "enabled")
+            client["enabled"] = config.getboolean(client_name,
+                                                  "enabled")
             
             client["fingerprint"] = (section["fingerprint"].upper()
                                      .replace(" ", ""))
@@ -497,14 +502,7 @@
             client["last_approval_request"] = None
             client["last_checked_ok"] = None
             client["last_checker_status"] = None
-            if client["enabled"]:
-                client["last_enabled"] = datetime.datetime.utcnow()
-                client["expires"] = (datetime.datetime.utcnow()
-                                     + client["timeout"])
-            else:
-                client["last_enabled"] = None
-                client["expires"] = None
-
+        
         return settings
         
         
@@ -517,12 +515,23 @@
         for setting, value in settings.iteritems():
             setattr(self, setting, value)
         
+        if self.enabled:
+            if not hasattr(self, "last_enabled"):
+                self.last_enabled = datetime.datetime.utcnow()
+            if not hasattr(self, "expires"):
+                self.expires = (datetime.datetime.utcnow()
+                                + self.timeout)
+        else:
+            self.last_enabled = None
+            self.expires = None
+       
         logger.debug("Creating client %r", self.name)
         # Uppercase and remove spaces from fingerprint for later
         # comparison purposes with return value from the fingerprint()
         # function
         logger.debug("  Fingerprint: %s", self.fingerprint)
-        self.created = settings.get("created", datetime.datetime.utcnow())
+        self.created = settings.get("created",
+                                    datetime.datetime.utcnow())
 
         # attributes specific for this server instance
         self.checker = None
@@ -1358,26 +1367,25 @@
         if value is None:       # get
             return dbus.UInt64(self.timeout_milliseconds())
         self.timeout = datetime.timedelta(0, 0, 0, value)
-        if getattr(self, "disable_initiator_tag", None) is None:
-            return
         # Reschedule timeout
-        gobject.source_remove(self.disable_initiator_tag)
-        self.disable_initiator_tag = None
-        self.expires = None
-        time_to_die = timedelta_to_milliseconds((self
-                                                 .last_checked_ok
-                                                 + self.timeout)
-                                                - datetime.datetime
-                                                .utcnow())
-        if time_to_die <= 0:
-            # The timeout has passed
-            self.disable()
-        else:
-            self.expires = (datetime.datetime.utcnow()
-                            + datetime.timedelta(milliseconds =
-                                                 time_to_die))
-            self.disable_initiator_tag = (gobject.timeout_add
-                                          (time_to_die, self.disable))
+        if self.enabled:
+            now = datetime.datetime.utcnow()
+            time_to_die = timedelta_to_milliseconds(
+                (self.last_checked_ok + self.timeout) - now)
+            if time_to_die <= 0:
+                # The timeout has passed
+                self.disable()
+            else:
+                self.expires = (now +
+                                datetime.timedelta(milliseconds =
+                                                   time_to_die))
+                if (getattr(self, "disable_initiator_tag", None)
+                    is None):
+                    return
+                gobject.source_remove(self.disable_initiator_tag)
+                self.disable_initiator_tag = (gobject.timeout_add
+                                              (time_to_die,
+                                               self.disable))
     
     # ExtendedTimeout - property
     @dbus_service_property(_interface, signature="t",
@@ -2094,7 +2102,8 @@
                                 % server_settings["servicename"]))
     
     # Parse config file with clients
-    client_config = configparser.SafeConfigParser(Client.client_defaults)
+    client_config = configparser.SafeConfigParser(Client
+                                                  .client_defaults)
     client_config.read(os.path.join(server_settings["configdir"],
                                     "clients.conf"))
     
@@ -2262,7 +2271,9 @@
                         client["expires"] = (datetime.datetime
                                              .utcnow()
                                              + client["timeout"])
-                    
+                        logger.debug("Last checker succeeded,"
+                                     " keeping {0} enabled"
+                                     .format(client["name"]))
             try:
                 client["secret"] = (
                     pgp.decrypt(client["encrypted_secret"],
@@ -2277,9 +2288,11 @@
 
     
     # Add/remove clients based on new changes made to config
-    for client_name in set(old_client_settings) - set(client_settings):
+    for client_name in (set(old_client_settings)
+                        - set(client_settings)):
         del clients_data[client_name]
-    for client_name in set(client_settings) - set(old_client_settings):
+    for client_name in (set(client_settings)
+                        - set(old_client_settings)):
         clients_data[client_name] = client_settings[client_name]
 
     # Create clients all clients

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2011-11-26 23:08:17 +0000
+++ mandos-clients.conf.xml	2011-12-31 23:05:34 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -36,6 +36,7 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-ctl'
--- mandos-ctl	2011-12-21 17:17:32 +0000
+++ mandos-ctl	2011-12-31 23:05:34 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'mandos-ctl.xml'
--- mandos-ctl.xml	2011-10-05 16:00:56 +0000
+++ mandos-ctl.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-ctl">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-keygen'
--- mandos-keygen	2011-10-15 16:48:03 +0000
+++ mandos-keygen	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 # 
 # Mandos key generator - create a new OpenPGP key for a Mandos client
 # 
-# Copyright © 2008-2011 Teddy Hogeborn
-# Copyright © 2008-2011 Björn Påhlsson
+# Copyright © 2008-2012 Teddy Hogeborn
+# Copyright © 2008-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2011-11-12 18:14:55 +0000
+++ mandos-keygen.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos-monitor'
--- mandos-monitor	2011-10-15 16:48:03 +0000
+++ mandos-monitor	2011-12-31 23:05:34 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2009-2011 Teddy Hogeborn
-# Copyright © 2009-2011 Björn Påhlsson
+# Copyright © 2009-2012 Teddy Hogeborn
+# Copyright © 2009-2012 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -242,6 +242,8 @@
         self.update()
     
     def checker_started(self, command):
+        """Server signals that a checker started. This could be useful
+           to log in the future. """
         #self.logger('Client %s started checker "%s"'
         #            % (self.properties["Name"], unicode(command)))
         pass
@@ -497,48 +499,6 @@
         
         self.busname = domain + '.Mandos'
         self.main_loop = gobject.MainLoop()
-        self.bus = dbus.SystemBus()
-        mandos_dbus_objc = self.bus.get_object(
-            self.busname, "/", follow_name_owner_changes=True)
-        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
-                                          dbus_interface
-                                          = server_interface)
-        try:
-            mandos_clients = (self.mandos_serv
-                              .GetAllClientsWithProperties())
-        except dbus.exceptions.DBusException:
-            mandos_clients = dbus.Dictionary()
-        
-        (self.mandos_serv
-         .connect_to_signal("ClientRemoved",
-                            self.find_and_remove_client,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientAdded",
-                            self.add_new_client,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        (self.mandos_serv
-         .connect_to_signal("ClientNotFound",
-                            self.client_not_found,
-                            dbus_interface=server_interface,
-                            byte_arrays=True))
-        for path, client in mandos_clients.iteritems():
-            client_proxy_object = self.bus.get_object(self.busname,
-                                                      path)
-            self.add_client(MandosClientWidget(server_proxy_object
-                                               =self.mandos_serv,
-                                               proxy_object
-                                               =client_proxy_object,
-                                               properties=client,
-                                               update_hook
-                                               =self.refresh,
-                                               delete_hook
-                                               =self.remove_client,
-                                               logger
-                                               =self.log_message),
-                            path=path)
     
     def client_not_found(self, fingerprint, address):
         self.log_message(("Client with address %s and fingerprint %s"
@@ -559,7 +519,6 @@
                                                      self.divider)))
         if self.log_visible:
             self.uilist.append(self.logbox)
-            pass
         self.topwidget = urwid.Pile(self.uilist)
     
     def log_message(self, message):
@@ -649,6 +608,49 @@
     
     def run(self):
         """Start the main loop and exit when it's done."""
+        self.bus = dbus.SystemBus()
+        mandos_dbus_objc = self.bus.get_object(
+            self.busname, "/", follow_name_owner_changes=True)
+        self.mandos_serv = dbus.Interface(mandos_dbus_objc,
+                                          dbus_interface
+                                          = server_interface)
+        try:
+            mandos_clients = (self.mandos_serv
+                              .GetAllClientsWithProperties())
+        except dbus.exceptions.DBusException:
+            mandos_clients = dbus.Dictionary()
+        
+        (self.mandos_serv
+         .connect_to_signal("ClientRemoved",
+                            self.find_and_remove_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientAdded",
+                            self.add_new_client,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        (self.mandos_serv
+         .connect_to_signal("ClientNotFound",
+                            self.client_not_found,
+                            dbus_interface=server_interface,
+                            byte_arrays=True))
+        for path, client in mandos_clients.iteritems():
+            client_proxy_object = self.bus.get_object(self.busname,
+                                                      path)
+            self.add_client(MandosClientWidget(server_proxy_object
+                                               =self.mandos_serv,
+                                               proxy_object
+                                               =client_proxy_object,
+                                               properties=client,
+                                               update_hook
+                                               =self.refresh,
+                                               delete_hook
+                                               =self.remove_client,
+                                               logger
+                                               =self.log_message),
+                            path=path)
+
         self.refresh()
         self._input_callback_tag = (gobject.io_add_watch
                                     (sys.stdin.fileno(),

=== modified file 'mandos-monitor.xml'
--- mandos-monitor.xml	2011-10-05 16:00:56 +0000
+++ mandos-monitor.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-monitor">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2011-11-26 22:22:20 +0000
+++ mandos.conf.xml	2011-12-31 23:05:34 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,6 +35,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'mandos.xml'
--- mandos.xml	2011-12-21 00:29:33 +0000
+++ mandos.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2011-11-26">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -35,6 +35,7 @@
       <year>2009</year>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	2011-12-27 03:56:39 +0000
+++ network-hooks.d/bridge	2011-12-31 14:05:57 +0000
@@ -15,13 +15,18 @@
 
 CONFIG="$MANDOSNETHOOKDIR/bridge.conf"
 
-# Read config file, which must set "BRIDGE", "PORTS", and optionally
-# "IPADDRS" and "ROUTES".
+addrtoif(){
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+}
+
+# Read config file, which must set "BRIDGE", "PORT_ADDRESSES", and
+# optionally "IPADDRS" and "ROUTES".
 if [ -e "$CONFIG" ]; then
     . "$CONFIG"
 fi
 
-if [ -z "$BRIDGE" -o -z "$PORTS" ]; then
+if [ -z "$BRIDGE" -o -z "$PORT_ADDRESSES" ]; then
     exit
 fi
 
@@ -39,9 +44,10 @@
 case "$1" in
     start)
 	"$brctl" addbr "$BRIDGE"
-	for port in $PORTS; do
-	    "$brctl" addif "$BRIDGE" "$port"
-	    ip link set dev "$port" up
+	for address in $PORT_ADDRESSES; do
+	    interface=`addrtoif "$address"`
+	    "$brctl" addif "$BRIDGE" "$interface"
+	    ip link set dev "$interface" up
 	done
 	ip link set dev "$BRIDGE" up
 	sleep "$DELAY"
@@ -58,9 +64,10 @@
 	;;
     stop)
 	ip link set dev "$BRIDGE" down
-	for port in $PORTS; do
-	    ip link set dev "$port" down
-	    "$brctl" delif "$BRIDGE" "$port"
+	for address in $PORT_ADDRESSES; do
+	    interface=`addrtoif "$address"`
+	    ip link set dev "$interface" down
+	    "$brctl" delif "$BRIDGE" "$interface"
 	done
 	"$brctl" delbr "$BRIDGE"
 	;;

=== modified file 'network-hooks.d/bridge.conf'
--- network-hooks.d/bridge.conf	2011-11-24 20:15:24 +0000
+++ network-hooks.d/bridge.conf	2011-12-31 13:25:58 +0000
@@ -2,7 +2,7 @@
 
 #BRIDGE=br0
 
-#PORTS="eth0 eth1"
+#PORT_ADDRESSES="00:11:22:33:44:55 11:22:33:44:55:66"
 
 ## Optional
 

=== modified file 'network-hooks.d/openvpn'
--- network-hooks.d/openvpn	2011-12-02 16:52:50 +0000
+++ network-hooks.d/openvpn	2011-12-31 14:05:57 +0000
@@ -16,7 +16,9 @@
 CONFIG="openvpn.conf"
 
 # Extract the "dev" setting from the config file
-VPNDEVICE="`sed -n -e 's/#.*//' -e 's/^[[:space:]]*dev[[:space:]]\+//p' \"$MANDOSNETHOOKDIR/$CONFIG\"`"
+VPNDEVICE=`sed -n -e 's/[[:space:]]#.*//' \
+    -e 's/^[[:space:]]*dev[[:space:]]\+//p' \
+    "$MANDOSNETHOOKDIR/$CONFIG"`
 
 PIDFILE=/run/openvpn-mandos.pid
 
@@ -34,7 +36,9 @@
 
 case "$1" in
     start)
-	"$openvpn" --cd "$MANDOSNETHOOKDIR" --daemon 'openvpn(Mandos)' --writepid "$PIDFILE" --config "$CONFIG"
+	"$openvpn" --cd "$MANDOSNETHOOKDIR" \
+	    --daemon 'openvpn(Mandos)' --writepid "$PIDFILE" \
+	    --config "$CONFIG"
 	sleep "$DELAY"
 	;;
     stop)

=== modified file 'network-hooks.d/wireless'
--- network-hooks.d/wireless	2011-12-30 17:14:15 +0000
+++ network-hooks.d/wireless	2011-12-31 14:05:57 +0000
@@ -20,6 +20,11 @@
 
 CONFIG="$MANDOSNETHOOKDIR/wireless.conf"
 
+addrtoif(){
+    grep -liFe "$1" /sys/class/net/*/address \
+	| sed -e 's,.*/\([^/]*\)/[^/]*,\1,'
+}
+
 # Read config file
 if [ -e "$CONFIG" ]; then
     . "$CONFIG"
@@ -27,13 +32,16 @@
     exit
 fi
 
-interfaces="`env|sed -n -e 's/^\(MODULE\|IPADDRS\|ROUTES\|WPA_DRIVER\)_\([^=]*\)=.*/\2/p' \"$CONFIG\" |sort -u`"
+ifkeys=`env | sed -n -e 's/^ADDRESS_\([^=]*\)=.*/\1/p' "$CONFIG" \
+    | sort -u`
 
 # Exit if DEVICE is set and is not any of the wireless interfaces
 if [ -n "$DEVICE" ]; then
     while :; do
-	for IF in $interfaces; do
-	    if [ "$IF" = "$DEVICE" ]; then
+	for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
+	    if [ "$INTERFACE" = "$DEVICE" ]; then
 		break 2
 	    fi
 	done
@@ -66,29 +74,40 @@
     start)
 	mkdir -m u=rwx,go= -p "$CTRLDIR"
 	"$wpa_supplicant" -B -g "$CTRL" -p "$CTRLDIR" $WPAS_OPTIONS
-        for INTERFACE in $interfaces; do
-	    DRIVER=`eval 'echo "$WPA_DRIVER_'"$INTERFACE"\"`
-	    DELAY=`eval 'echo "$DELAY_'"$INTERFACE"\"`
+        for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
+	    DRIVER=`eval 'echo "$WPA_DRIVER_'"$KEY"\"`
+	    IFDELAY=`eval 'echo "$DELAY_'"$KEY"\"`
 	    "$wpa_cli" -g "$CTRL" interface_add "$INTERFACE" "" \
 		"${DRIVER:-wext}" "$CTRLDIR" > /dev/null \
 		| sed -e '/^OK$/d'
-	    NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" add_network`
-	    eval wpa_interface_"$INTERFACE"
+	    NETWORK=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" \
+		add_network`
+	    eval wpa_interface_"$KEY"
 	    "$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" enable_network \
 		"$NETWORK" | sed -e '/^OK$/d'
-	    ( sleep "${DELAY:-$DELAY}" || : ) &
+	    sleep "${IFDELAY:-$DELAY}" &
 	    sleep=$!
 	    while :; do
-		kill -0 $sleep || break
-		STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" status | sed -n -e 's/^wpa_state=//p'`
+		kill -0 $sleep 2>/dev/null || break
+		STATE=`"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" \
+		    status | sed -n -e 's/^wpa_state=//p'`
 		if [ "$STATE" = COMPLETED ]; then
-		    kill $sleep
-		    break
+		    while :; do
+			kill -0 $sleep 2>/dev/null || break 2
+			UP=`cat /sys/class/net/"$INTERFACE"/operstate`
+			if [ "$UP" = up ]; then
+			    kill $sleep 2>/dev/null
+			    break 2
+			fi
+			sleep 1
+		    done
 		fi
 		sleep 1
 	    done &
 	    wait $sleep || :
-	    IPADDRS=`eval 'echo "$IPADDRS_'"$INTERFACE"\"`
+	    IPADDRS=`eval 'echo "$IPADDRS_'"$KEY"\"`
 	    if [ -n "$IPADDRS" ]; then
 		if [ "$IPADDRS" = dhcp ]; then
 		    ipconfig -c dhcp -d "$INTERFACE" || :
@@ -99,7 +118,7 @@
 		    done
 		fi
 	    fi
-	    ROUTES=`eval 'echo "$ROUTES_'"$INTERFACE"\"`
+	    ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
 	    if [ -n "$ROUTES" ]; then
 		for route in $ROUTES; do
 		    "$ip" route add "$route" dev "$BRIDGE"
@@ -109,7 +128,9 @@
 	;;
     stop)
 	"$wpa_cli" -g "$CTRL" terminate 2>&1 | sed -e '/^OK$/d'
-        for INTERFACE in $interfaces; do
+        for KEY in $ifkeys; do
+	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
+	    INTERFACE=`addrtoif "$ADDRESS"`
 	    "$ip" addr show scope global permanent dev "$INTERFACE" \
 		| while read type addr rest; do
 		case "$type" in
@@ -130,7 +151,7 @@
 	if [ "$IPADDRS" = dhcp ]; then
 	    echo af_packet
 	fi
-	sed -n -e 's/#.*$//' -e 's/[ 	]*$//' -e 's/^MODULE=//p' \
-	    "$CONFIG"
+	sed -n -e 's/#.*$//' -e 's/[ 	]*$//' \
+	    -e 's/^MODULE_[^=]\+=//p' "$CONFIG"
 	;;
 esac

=== modified file 'network-hooks.d/wireless.conf'
--- network-hooks.d/wireless.conf	2011-12-27 03:56:39 +0000
+++ network-hooks.d/wireless.conf	2011-12-31 13:25:58 +0000
@@ -2,16 +2,22 @@
 #WPAS_OPTIONS=""
 
 # wlan0
-MODULE_wlan0=ath9k
-#WPA_DRIVER_wlan0=wext
-wpa_interface_wlan0(){
+ADDRESS_0=00:11:22:33:44:55
+MODULE_0=ath9k
+#WPA_DRIVER_0=wext
+wpa_interface_0(){
     # Use this format to set simple things:
     wpa_cli_set ssid home
     wpa_cli_set psk "secret passphrase"
     # Use this format to do more complex things with wpa_cli:
-    #"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" set_network "$NETWORK" passphrase '"secret passphrase"'
+    #"$wpa_cli" -p "$CTRLDIR" -i "$INTERFACE" bssid "$NETWORK" 00:11:22:33:44:55
+    #"$wpa_cli" -g "$CTRL" ping
 }
-#DELAY_wlan0=10
-IPADDRS_wlan0=dhcp
-#IPADDRS_wlan0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
-#ROUTES_wlan0="192.0.2.0/24 2001:DB8::/32"
+#DELAY_0=10
+IPADDRS_0=dhcp
+#IPADDRS_0="192.0.2.3/24 2001:DB8::aede:48ff:fe71:f6f2/32"
+#ROUTES_0="192.0.2.0/24 2001:DB8::/32"
+
+#ADDRESS_1=11:22:33:44:55:66
+#MODULE_1=...
+#...

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2011-12-25 00:40:09 +0000
+++ plugin-runner.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -426,7 +426,7 @@
     errno = 0;
     switch(key){
       char *tmp;
-      intmax_t tmpmax;
+      intmax_t tmp_id;
     case 'g': 			/* --global-options */
       {
 	char *plugin_option;
@@ -505,24 +505,24 @@
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      tmpmax = strtoimax(arg, &tmp, 10);
+      tmp_id = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmpmax != (uid_t)tmpmax){
+	 or tmp_id != (uid_t)tmp_id){
 	argp_error(state, "Bad user ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)uid);
 	break;
       }
-      uid = (uid_t)tmpmax;
+      uid = (uid_t)tmp_id;
       break;
     case 131:			/* --groupid */
-      tmpmax = strtoimax(arg, &tmp, 10);
+      tmp_id = strtoimax(arg, &tmp, 10);
       if(errno != 0 or tmp == arg or *tmp != '\0'
-	 or tmpmax != (gid_t)tmpmax){
+	 or tmp_id != (gid_t)tmp_id){
 	argp_error(state, "Bad group ID number: \"%s\", using %"
 		   PRIdMAX, arg, (intmax_t)gid);
 	break;
       }
-      gid = (gid_t)tmpmax;
+      gid = (gid_t)tmp_id;
       break;
     case 132:			/* --debug */
       debug = true;

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2011-10-05 16:56:06 +0000
+++ plugin-runner.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2011-10-05">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	2011-12-24 23:17:02 +0000
+++ plugins.d/askpass-fifo.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Askpass-FIFO - Read a password from a FIFO and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/askpass-fifo.xml'
--- plugins.d/askpass-fifo.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/askpass-fifo.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "askpass-fifo">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2011-12-30 17:14:15 +0000
+++ plugins.d/mandos-client.c	2011-12-31 23:05:34 +0000
@@ -9,8 +9,8 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -1474,7 +1474,7 @@
 	}
 	free(delaystring);
 	if(connect_to != NULL){
-	  ret = setenv("CONNECT", connect_to);
+	  ret = setenv("CONNECT", connect_to, 1);
 	  if(ret == -1){
 	    perror_plus("setenv");
 	    _exit(EX_OSERR);

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2011-12-27 03:56:39 +0000
+++ plugins.d/mandos-client.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2011-11-27">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2011-12-25 00:40:09 +0000
+++ plugins.d/password-prompt.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Password-prompt - Read a password from the terminal and print it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -124,13 +124,13 @@
     int ret;
     int cl_fd;
     {
-      uintmax_t maxvalue;
+      uintmax_t proc_id;
       char *tmp;
       errno = 0;
-      maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
+      proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
       
       if(errno != 0 or *tmp != '\0'
-	 or maxvalue != (uintmax_t)((pid_t)maxvalue)){
+	 or proc_id != (uintmax_t)((pid_t)proc_id)){
 	return 0;
       }
     }
@@ -514,6 +514,7 @@
 	switch(e){
 	case EBADF:
 	  status = EX_UNAVAILABLE;
+	  break;
 	case EIO:
 	case EINVAL:
 	default:

=== modified file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/password-prompt.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "password-prompt">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/plymouth.c'
--- plugins.d/plymouth.c	2011-12-25 00:40:09 +0000
+++ plugins.d/plymouth.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Plymouth - Read a password from Plymouth and output it
  * 
- * Copyright © 2010-2011 Teddy Hogeborn
- * Copyright © 2010-2011 Björn Påhlsson
+ * Copyright © 2010-2012 Teddy Hogeborn
+ * Copyright © 2010-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -218,13 +218,13 @@
 int is_plymouth(const struct dirent *proc_entry){
   int ret;
   {
-    uintmax_t maxvalue;
+    uintmax_t proc_id;
     char *tmp;
     errno = 0;
-    maxvalue = strtoumax(proc_entry->d_name, &tmp, 10);
+    proc_id = strtoumax(proc_entry->d_name, &tmp, 10);
 
     if(errno != 0 or *tmp != '\0'
-       or maxvalue != (uintmax_t)((pid_t)maxvalue)){
+       or proc_id != (uintmax_t)((pid_t)proc_id)){
       return 0;
     }
   }
@@ -265,36 +265,36 @@
 
 pid_t get_pid(void){
   int ret;
-  uintmax_t maxvalue = 0;
+  uintmax_t proc_id = 0;
   FILE *pidfile = fopen(plymouth_pid, "r");
   /* Try the new pid file location */
   if(pidfile != NULL){
-    ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+    ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
     if(ret != 1){
-      maxvalue = 0;
+      proc_id = 0;
     }
     fclose(pidfile);
   }
   /* Try the old pid file location */
-  if(maxvalue == 0){
+  if(proc_id == 0){
     pidfile = fopen(plymouth_pid, "r");
     if(pidfile != NULL){
-      ret = fscanf(pidfile, "%" SCNuMAX, &maxvalue);
+      ret = fscanf(pidfile, "%" SCNuMAX, &proc_id);
       if(ret != 1){
-	maxvalue = 0;
+	proc_id = 0;
       }
       fclose(pidfile);
     }
   }
   /* Look for a plymouth process */
-  if(maxvalue == 0){
+  if(proc_id == 0){
     struct dirent **direntries = NULL;
     ret = scandir("/proc", &direntries, is_plymouth, alphasort);
     if (ret == -1){
       error_plus(0, errno, "scandir");
     }
     if (ret > 0){
-      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &maxvalue);
+      ret = sscanf(direntries[0]->d_name, "%" SCNuMAX, &proc_id);
       if (ret < 0){
 	error_plus(0, errno, "sscanf");
       }
@@ -304,8 +304,8 @@
     free(direntries);
   }
   pid_t pid;
-  pid = (pid_t)maxvalue;
-  if((uintmax_t)pid == maxvalue){
+  pid = (pid_t)proc_id;
+  if((uintmax_t)pid == proc_id){
     return pid;
   }
   

=== modified file 'plugins.d/plymouth.xml'
--- plugins.d/plymouth.xml	2011-11-12 18:14:55 +0000
+++ plugins.d/plymouth.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plymouth">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     <copyright>
       <year>2010</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	2011-12-24 23:17:02 +0000
+++ plugins.d/splashy.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Splashy - Read a password from splashy and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/splashy.xml'
--- plugins.d/splashy.xml	2011-10-05 16:00:56 +0000
+++ plugins.d/splashy.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "splashy">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -33,7 +33,7 @@
     <copyright>
       <year>2008</year>
       <year>2009</year>
-      <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	2011-12-24 23:17:02 +0000
+++ plugins.d/usplash.c	2011-12-31 23:05:34 +0000
@@ -2,8 +2,8 @@
 /*
  * Usplash - Read a password from usplash and output it
  * 
- * Copyright © 2008-2011 Teddy Hogeborn
- * Copyright © 2008-2011 Björn Påhlsson
+ * Copyright © 2008-2012 Teddy Hogeborn
+ * Copyright © 2008-2012 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as

=== modified file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	2011-11-12 18:14:55 +0000
+++ plugins.d/usplash.xml	2011-12-31 23:05:34 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "usplash">
-<!ENTITY TIMESTAMP "2011-10-03">
+<!ENTITY TIMESTAMP "2012-01-01">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2011</year>
+      <year>2012</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

